# -*- encoding: utf-8 -*-
'''
@File    :   __init__.py
@Author  :   北极星光 
@Contact :   light22@126.com
'''

from .excel_operate import ExcelOperate
from .sheet_copy import SheetCopy
from .sheet_comparison import SheetComparison
from .list_operate import *



# -*- encoding: utf-8 -*-
'''
@File    :   sheet_copy.py
@Author  :   北极星光 
@Contact :   light22@126.com
'''

from excel_operate import ExcelOperate
from copy import copy
from openpyxl.utils import get_column_letter


class SheetCopy:
    def __init__(self, src_file_path: str = None, tag_file_path: str = None, sheet_name: str = None, column_adjust: float = 0) -> None:
        '''===================================\n
        src_file_path: 此参数为源Excel文件路径
        tag_file_path: 此参数为目标Excel文件路径，不指定的情况下默认新建工作薄
        sheet_name: 此参数为被复制的工作表名称，不指定的情况下默认为源工作表中的当前激活工作表
        column_adjust: 此参数为列宽修正系数，作用为修正列宽误差
        '''
        self.src_file = ExcelOperate(src_file_path, sheet_name)
        if tag_file_path == None:
            self.tag_file = ExcelOperate(sheet_name=self.src_file.ws.title)
        else:
            self.tag_file = ExcelOperate(tag_file_path)
            if self.src_file.ws.title not in self.tag_file.wb.sheetnames:
                self.tag_file.wb.create_sheet(self.src_file.ws.title)
                self.tag_file.ws = self.tag_file.wb[self.src_file.ws.title]
            else:
                n = 1
                while True:
                    if f'{self.src_file.ws.title} ({n})' not in self.tag_file.wb.sheetnames:
                        self.tag_file.wb.create_sheet(
                            f'{self.src_file.ws.title} ({n})')
                        self.tag_file.ws = self.tag_file.wb[
                            f'{self.src_file.ws.title} ({n})']
                        break
                    n += 1
        # 列宽误差修正系数，默认为0，由于openpyxl在设置列宽时与实际存在出入，因此可以给定此参数来调整误差
        self.column_adjust = column_adjust

    # 定义工作表复制模块
    def copy_sheet(self, src_file: ExcelOperate = None, tag_file: ExcelOperate = None, origin_row: int = 1, origin_col: int = 1):
        '''===================================\n
        复制工作表
        src_file: 原文件的ExcelOperate对象
        tag_file: 目标文件的ExcelOperate对象
        origin_row: 设定目标工作表写入的起始行，默认从第1行开始写入，传入大于或等于1的整数则会从设定的行开始写入。
        origin_col: 设定目标工作表写入的起始列，默认从第1列开始写入，传入大于或等于1的整数则会从设定的列开始写入。
        '''
        if src_file == None:
            src_file = self.src_file
        if tag_file == None:
            tag_file = self.tag_file
        for row in src_file.ws:
            # 遍历源xlsx文件制定sheet中的所有单元格
            for cell in row:  # 复制数据
                cell.row += origin_row - 1
                cell.column += origin_col - 1
                tag_file.ws[cell.coordinate].value = cell.value
                if cell.has_style:  # 复制样式
                    tag_file.ws[cell.coordinate].font = copy(cell.font)
                    tag_file.ws[cell.coordinate].border = copy(cell.border)
                    tag_file.ws[cell.coordinate].fill = copy(cell.fill)
                    tag_file.ws[cell.coordinate].number_format = copy(
                        cell.number_format)
                    tag_file.ws[cell.coordinate].protection = copy(
                        cell.protection)
                    tag_file.ws[cell.coordinate].alignment = copy(
                        cell.alignment)

        merged_cells = src_file.ws.merged_cells.ranges  # 已合并的单元格列表
        if len(merged_cells) > 0:  # 检测源xlsx中合并的单元格
            for merged_cell in merged_cells:
                tag_file.ws.merge_cells(start_row=merged_cell.min_row + origin_row - 1, end_row=merged_cell.max_row + origin_row - 1,
                                        start_column=merged_cell.min_col + origin_col - 1, end_column=merged_cell.max_col + origin_col - 1)  # 合并单元格
        # 开始处理行高列宽
        for i in range(1, src_file.ws.max_row + 1):
            tag_file.ws.row_dimensions[i + origin_row -
                                       1].height = src_file.ws.row_dimensions[i].height
        for i in range(1, src_file.ws.max_column + 1):
            tag_file.ws.column_dimensions[get_column_letter(
                i + origin_col - 1)].width = src_file.ws.column_dimensions[get_column_letter(i)].width + self.column_adjust  # 修正列宽误差

        return tag_file


# 调试
if __name__ == '__main__':
    src_file_path = r'D:\codes\Python Projects\Excel-operate\tests\示例.xlsx'
    copyer = SheetCopy(src_file_path)
    tag_file = copyer.copy_sheet(origin_col=5, origin_row=8)
    tag_file.save(r'D:\Desktop\1111.xlsx')



# -*- encoding: utf-8 -*-
'''
@File    :   excel_operate.py
@Author  :   北极星光 
@Contact :   light22@126.com
'''

from openpyxl import Workbook, load_workbook
from copy import copy
from openpyxl.utils import get_column_letter


class ExcelOperate:
    def __init__(self, file_path: str = None, sheet_name: str = None):
        '''===================================\n
        实例化Excel对象
        file_path: 指定目标Excel文件路径，不指定的话则默认打开新建工作薄
        sheet_name: 指定目标工作表名称，不指定的话默认为当前激活工作表，如果当前工作薄中不含有指定的工作表则会新建对应名称的工作表
        '''
        self.file_path = file_path
        if file_path == None:
            self.wb = Workbook()
        else:
            self.wb = load_workbook(file_path)

        if sheet_name == None:
            self.ws = self.wb.active
        else:
            if file_path == None:
                self.ws = self.wb.active
                self.ws.title = sheet_name
            else:
                if sheet_name not in self.wb.sheetnames:
                    self.wb.create_sheet(sheet_name)
                self.ws = self.wb[sheet_name]

    def insert_rows(self, idx: int, amount: int = 1, height: float = 'before'):  # 定义插入行模块
        '''===================================\n
        插入行
        idx: 参数idx为指定插入行的起始位置
        amount: 参数amount为指定插入的行数，不指定的情况下默认插入1行
        height: 参数height为指定插入行的行高，不指定的情况下默认继承前一行的行高，指定数字可以设置对应的行高，指定None为自动行高
        '''
        row_height_list = []  # 定义空列表用以接收从插入行开始到最大行的所有行高数据
        for i in range(idx, self.ws.max_row+1):
            row_height_list.append(self.ws.row_dimensions[i].height)

        merged_cells = self.ws.merged_cell_ranges  # 当前工作表中所有已合并的单元格列表
        if len(merged_cells) > 0:  # 判断是否存在合并单元格
            merged_cell_list = []  # 定义空列表，方便储存插入行后的已合并单元格信息
            for merged_cell in merged_cells:
                merged_cell_info = {}
                if merged_cell.min_row >= idx:
                    merged_cell_info['min_row'] = merged_cell.min_row + amount
                else:
                    merged_cell_info['min_row'] = merged_cell.min_row

                if merged_cell.max_row >= idx:
                    merged_cell_info['max_row'] = merged_cell.max_row + amount
                else:
                    merged_cell_info['max_row'] = merged_cell.max_row

                merged_cell_info['min_col'] = merged_cell.min_col
                merged_cell_info['max_col'] = merged_cell.max_col

                merged_cell_list.append(merged_cell_info)  # 将插入行后的合并单元格信息存入列表

                self.ws.unmerge_cells(start_row=merged_cell.min_row, end_row=merged_cell.max_row,
                                      start_column=merged_cell.min_col, end_column=merged_cell.max_col)  # 解除合并单元格

            self.ws.insert_rows(idx, amount)  # 插入空白行

            for i in merged_cell_list:
                self.ws.merge_cells(start_row=i['min_row'], end_row=i['max_row'],
                                    start_column=i['min_col'], end_column=i['max_col'])  # 重新合并单元格

        else:
            self.ws.insert_rows(idx, amount)  # 插入空白行

        # 从插入行的上方复制单元格格式
        for src_row in self.ws.iter_rows(min_row=idx-1, max_row=idx-1):
            for tag_row in self.ws.iter_rows(min_row=idx, max_row=idx+amount-1):
                for src_cell, tag_cell in zip(src_row, tag_row):
                    tag_cell.font = copy(src_cell.font)  # 复制字体
                    tag_cell.border = copy(src_cell.border)  # 复制边框
                    tag_cell.fill = copy(src_cell.fill)  # 复制填充色
                    tag_cell.number_format = copy(
                        src_cell.number_format)  # 复制数据格式
                    tag_cell.protection = copy(src_cell.protection)  # 复制保护状态
                    tag_cell.alignment = copy(src_cell.alignment)  # 复制对齐格式

        for i in range(idx, idx+amount+1):  # 设置插入行的行高
            if height == 'before':
                self.ws.row_dimensions[i].height = self.ws.row_dimensions[idx-1].height
            else:
                self.ws.row_dimensions[i].height = height

        for i, h in zip(range(idx+amount, self.ws.max_row+1), row_height_list):
            self.ws.row_dimensions[i].height = h

    def insert_cols(self, idx: int, amount: int = 1, width: float = 'before'):  # 定义插入列模块
        '''===================================\n
        插入列
        idx: 参数idx为指定插入列起始的位置
        amount: 参数amount为指定插入的列数，不指定的情况下默认插入1列
        width: 参数width为指定插入列的列宽，不指定的情况下默认继承前一列的列宽，指定数字可以设置对应的列宽
        '''
        col_width_list = []  # 定义空列表用以接收从插入列开始到最大列的所有列宽数据
        for i in range(idx, self.ws.max_column+1):
            col_width_list.append(
                self.ws.column_dimensions[get_column_letter(i)].width)

        merged_cells = self.ws.merged_cell_ranges  # 当前工作表中所有已合并的单元格列表
        if len(merged_cells) > 0:  # 判断是否存在合并单元格
            merged_cell_list = []  # 定义空列表，方便储存插入列后的已合并单元格信息
            for merged_cell in merged_cells:
                merged_cell_info = {}
                if merged_cell.min_col >= idx:
                    merged_cell_info['min_col'] = merged_cell.min_col + amount
                else:
                    merged_cell_info['min_col'] = merged_cell.min_col

                if merged_cell.max_col >= idx:
                    merged_cell_info['max_col'] = merged_cell.max_col + amount
                else:
                    merged_cell_info['max_col'] = merged_cell.max_col

                merged_cell_info['min_row'] = merged_cell.min_row
                merged_cell_info['max_row'] = merged_cell.max_row

                merged_cell_list.append(merged_cell_info)  # 将插入列后的合并单元格信息存入列表

                self.ws.unmerge_cells(start_row=merged_cell.min_row, end_row=merged_cell.max_row,
                                      start_column=merged_cell.min_col, end_column=merged_cell.max_col)  # 解除合并单元格

            self.ws.insert_cols(idx, amount)  # 插入空白列

            for i in merged_cell_list:
                self.ws.merge_cells(start_row=i['min_row'], end_row=i['max_row'],
                                    start_column=i['min_col'], end_column=i['max_col'])  # 重新合并单元格

        else:
            self.ws.insert_cols(idx, amount)  # 插入空白列

        # 从插入列的前方复制单元格格式
        for src_col in self.ws.iter_cols(min_col=idx-1, max_col=idx-1):
            for tag_col in self.ws.iter_cols(min_col=idx, max_col=idx+amount-1):
                for src_cell, tag_cell in zip(src_col, tag_col):
                    tag_cell.font = copy(src_cell.font)  # 复制字体
                    tag_cell.border = copy(src_cell.border)  # 复制边框
                    tag_cell.fill = copy(src_cell.fill)  # 复制填充色
                    tag_cell.number_format = copy(
                        src_cell.number_format)  # 复制数据格式
                    tag_cell.protection = copy(src_cell.protection)  # 复制保护状态
                    tag_cell.alignment = copy(src_cell.alignment)  # 复制对齐格式

        for i in range(idx, idx+amount+1):  # 设置插入列的列宽
            if width == 'before':
                self.ws.column_dimensions[get_column_letter(
                    i)].width = self.ws.column_dimensions[get_column_letter(idx-1)].width
            else:
                self.ws.column_dimensions[get_column_letter(i)].width = width

        for i, w in zip(range(idx+amount, self.ws.max_column+1), col_width_list):
            self.ws.column_dimensions[get_column_letter(i)].width = w

    def delete_rows(self, idx: int, amount: int = 1):  # 定义删除行模块
        '''===================================\n
        删除行
        idx: 参数idx为指定删除行的起始位置
        amount: 参数amount为指定删除的行数，不指定的情况下默认删除1行
        '''
        row_height_list = []  # 定义空列表用以接收从删除行结尾开始到最大行的所有行高数据
        for i in range(idx+amount, self.ws.max_row+1):
            row_height_list.append(self.ws.row_dimensions[i].height)

        merged_cells = self.ws.merged_cell_ranges  # 当前工作表中所有已合并的单元格列表
        if len(merged_cells) > 0:  # 判断是否存在合并单元格
            merged_cell_list = []  # 定义空列表，方便储存删除行后的已合并单元格信息
            for merged_cell in merged_cells:
                merged_cell_info = {}
                if merged_cell.min_row >= idx and merged_cell.max_row <= idx + amount - 1:
                    self.ws.unmerge_cells(start_row=merged_cell.min_row, end_row=merged_cell.max_row,
                                          start_column=merged_cell.min_col, end_column=merged_cell.max_col)  # 解除合并单元格
                    continue
                if merged_cell.min_row >= idx:
                    if merged_cell.min_row - amount < idx:
                        merged_cell_info['min_row'] = idx
                    else:
                        merged_cell_info['min_row'] = merged_cell.min_row - amount
                else:
                    merged_cell_info['min_row'] = merged_cell.min_row

                if merged_cell.max_row >= idx:
                    if merged_cell.max_row - amount < idx - 1:
                        merged_cell_info['max_row'] = idx - 1
                    else:
                        merged_cell_info['max_row'] = merged_cell.max_row - amount
                else:
                    merged_cell_info['max_row'] = merged_cell.max_row

                merged_cell_info['min_col'] = merged_cell.min_col
                merged_cell_info['max_col'] = merged_cell.max_col

                merged_cell_list.append(merged_cell_info)  # 将删除行后的合并单元格信息存入列表

                self.ws.unmerge_cells(start_row=merged_cell.min_row, end_row=merged_cell.max_row,
                                      start_column=merged_cell.min_col, end_column=merged_cell.max_col)  # 解除合并单元格

            self.ws.delete_rows(idx, amount)  # 删除行

            for i in merged_cell_list:
                self.ws.merge_cells(start_row=i['min_row'], end_row=i['max_row'],
                                    start_column=i['min_col'], end_column=i['max_col'])  # 重新合并单元格

        else:
            self.ws.delete_rows(idx, amount)  # 删除行

        for i, h in zip(range(idx, self.ws.max_row+1), row_height_list):  # 恢复删除行后的行高
            self.ws.row_dimensions[i].height = h

    def delete_cols(self, idx: int, amount: int = 1):  # 定义删除列模块
        '''===================================\n
        删除列
        idx: 参数idx为指定删除列的起始位置
        amount: 参数amount为指定删除的列数，不指定的情况下默认删除1列
        '''
        col_width_list = []  # 定义空列表用以接收从删除列结尾开始到最大列的所有列宽数据
        for i in range(idx+amount, self.ws.max_column+1):
            col_width_list.append(
                self.ws.column_dimensions[get_column_letter(i)].width)

        merged_cells = self.ws.merged_cell_ranges  # 当前工作表中所有已合并的单元格列表
        if len(merged_cells) > 0:  # 判断是否存在合并单元格
            merged_cell_list = []  # 定义空列表，方便储存删除列后的已合并单元格信息
            for merged_cell in merged_cells:
                merged_cell_info = {}
                if merged_cell.min_col >= idx and merged_cell.max_col <= idx + amount - 1:
                    self.ws.unmerge_cells(start_row=merged_cell.min_row, end_row=merged_cell.max_row,
                                          start_column=merged_cell.min_col, end_column=merged_cell.max_col)  # 解除合并单元格
                    continue
                if merged_cell.min_col >= idx:
                    if merged_cell.min_col - amount < idx:
                        merged_cell_info['min_col'] = idx
                    else:
                        merged_cell_info['min_col'] = merged_cell.min_col - amount
                else:
                    merged_cell_info['min_col'] = merged_cell.min_col

                if merged_cell.max_col >= idx:
                    if merged_cell.max_col - amount < idx - 1:
                        merged_cell_info['max_col'] = idx - 1
                    else:
                        merged_cell_info['max_col'] = merged_cell.max_col - amount
                else:
                    merged_cell_info['max_col'] = merged_cell.max_col

                merged_cell_info['min_row'] = merged_cell.min_row
                merged_cell_info['max_row'] = merged_cell.max_row

                merged_cell_list.append(merged_cell_info)  # 将插入列后的合并单元格信息存入列表

                self.ws.unmerge_cells(start_row=merged_cell.min_row, end_row=merged_cell.max_row,
                                      start_column=merged_cell.min_col, end_column=merged_cell.max_col)  # 解除合并单元格

            self.ws.delete_cols(idx, amount)  # 删除列

            for i in merged_cell_list:
                self.ws.merge_cells(start_row=i['min_row'], end_row=i['max_row'],
                                    start_column=i['min_col'], end_column=i['max_col'])  # 重新合并单元格

        else:
            self.ws.delete_cols(idx, amount)  # 删除列

        for i, w in zip(range(idx, self.ws.max_column+1), col_width_list):  # 恢复删除行后的列宽
            self.ws.column_dimensions[get_column_letter(i)].width = w

    def save(self, file_path: str = None):
        '''===================================\n
        指定路径，保存Excel文件
        file_path: 指定文件保存路径，如果对象为已有工作薄的话，此参数不指定则默认为覆盖原文件保存，如果对象为新建工作薄的话此参数必须指定，否则会报错
        '''
        if file_path == None:
            self.wb.save(self.file_path)
        else:
            self.wb.save(file_path)


# 调试
if __name__ == '__main__':
    excel = ExcelOperate(sheet_name='测试工作表')

    excel.ws['C15'].value = '测试通过！！'
    excel.save('d:/desktop/1111.xlsx')



# -*- encoding: utf-8 -*-
'''
@File    :   sheet_comparison.py
@Author  :   北极星光 
@Contact :   light22@126.com
'''

from excel_operate import ExcelOperate
from sheet_copy import SheetCopy
from list_operate import *
from openpyxl.utils import get_column_letter
from openpyxl.styles import PatternFill, Alignment, Side, Border, Font


class SheetComparison:
    def __init__(self, src_excel: ExcelOperate, cmp_excel: ExcelOperate) -> None:
        '''===================================\n
        传入两个Excel对象，进行比较。生成报告
        src_excel: 原Excel对象，可传入ExcelOperate类对象
        cmp_excel: 待比较的Excel对象，可传入ExcelOperate类对象
        '''
        self.src_excel = src_excel
        self.cmp_excel = cmp_excel
        self.report_path = report_path

    def set_title_row(self, src_title_row: int, cmp_title_row: int):
        '''===================================\n
        此方法可以为原工作表及待比较工作表设置表头行
        src_title_row: 原Excel工作表的表头行，可传入整数类型
        cmp_title_row: 待比较Excel工作表的表头行，可传入整数类型
        '''
        self.src_title_row = src_title_row
        self.cmp_title_row = cmp_title_row

    def set_key_col(self, src_key_col: int, cmp_key_col: int):
        '''===================================\n
        此方法可以为原工作表及待比较工作表设置关键列
        src_key_col: 原Excel工作表的关键列，可传入整数类型
        cmp_key_col: 待比较Excel工作表的关键列，可传入整数类型
        '''
        self.src_key_col = src_key_col
        self.cmp_key_col = cmp_key_col

    def font_color(self, cell, style):
        '''===================================\n
        此方法为目标单元格设置样式
        cell: 目标单元格，请传入单元格对象
        style: 目标单元格将要设置成的样式，只支持'zeng','shan'和'gai'这三个参数。
        '''
        if style == 'zeng':
            cell.font = Font(color='0000FF')
        if style == 'shan':
            cell.font = Font(color='FF0000', strike=True)
        if style == 'gai':
            cell.font = Font(color='FF00FF')

    def compare(self):
        '''===================================\n
        对比工作表：将原工作表及目标工作表的表头行和关键列设置好之后即可使用此方法对比工作表，并返回对比报告。
        '''
        # 如果目标工作表的表头行与原工作表的表头行不在同一行
        if self.src_title_row != self.cmp_title_row:
            if self.src_title_row > self.cmp_title_row:
                self.cmp_excel.insert_rows(
                    1, self.src_title_row-self.cmp_title_row)
                self.cmp_title_row = self.src_title_row
            else:
                self.src_excel.insert_rows(
                    1, self.cmp_title_row-self.src_title_row)
                self.src_title_row = self.cmp_title_row

        # 如果目标工作表的关键列与原工作表的关键列不在同一列
        if self.src_key_col != self.cmp_key_col:
            if self.src_key_col > self.cmp_key_col:
                self.cmp_excel.insert_rows(
                    1, self.src_key_col-self.cmp_key_col)
                self.cmp_key_col = self.src_key_col
            else:
                self.src_excel.insert_rows(
                    1, self.cmp_key_col-self.src_key_col)
                self.src_key_col = self.cmp_key_col

        # 如果目标工作表的表头行与原工作表的表头行不相同
        src_title_list = [
            i.value for i in self.src_excel.ws[self.src_title_row]]
        cmp_title_list = [
            i.value for i in self.cmp_excel.ws[self.cmp_title_row]]
        if src_title_list != cmp_title_list:
            self.src_excel.insert_rows(1)
            self.src_title_row += 1
            self.cmp_excel.insert_rows(1)
            self.cmp_title_row += 1
            src_title_list = duplicate_to_only(
                src_title_list)  # 通过重命名的方式给列表去重。
            cmp_title_list = duplicate_to_only(
                cmp_title_list)  # 通过重命名的方式给列表去重。
            title_insert_info = is_insert(src_title_list, cmp_title_list)
            title_delete_info = is_insert(cmp_title_list, src_title_list)
            title_append_info = is_appand(src_title_list, cmp_title_list)
            if title_insert_info != None:
                n = 1
                for i in title_insert_info:
                    self.src_excel.insert_cols(i + n, title_insert_info[i])
                    n += title_insert_info[i]
                src_title_list
            if title_delete_info != None:
                n = 1
                for i in title_delete_info:
                    self.cmp_excel.insert_cols(i + n, title_delete_info[i])
                    n += title_delete_info[i]
            if title_append_info != None:
                self.src_excel.insert_cols(
                    self.src_excel.ws.max_column + 1, title_append_info)
            src_title_list = [
                i.value for i in self.src_excel.ws[self.src_title_row]]
            cmp_title_list = [
                i.value for i in self.cmp_excel.ws[self.cmp_title_row]]
            for idx, i, j in zip(range(len(src_title_list)), src_title_list, cmp_title_list):
                if i == None:
                    self.src_excel.ws[1][idx].value = '临'
                    self.cmp_excel.ws[1][idx].value = '增'
                if j == None:
                    self.src_excel.ws[1][idx].value = '删'
                    self.cmp_excel.ws[1][idx].value = '临'

            # 设置对齐、边框及字体颜色
            for cell in self.src_excel.ws[1]:
                cell.alignment = Alignment(
                    horizontal='center', vertical='center')
                cell.border = Border(left=Side('thin'), right=Side(
                    'thin'), top=Side('thin'), bottom=Side('thin'))
            for cell in self.cmp_excel.ws[1]:
                cell.alignment = Alignment(
                    horizontal='center', vertical='center')
                cell.border = Border(left=Side('thin'), right=Side(
                    'thin'), top=Side('thin'), bottom=Side('thin'))
            for col in self.src_excel.ws.iter_cols():
                if col[0].value == '删':
                    for cell in col:
                        self.font_color(cell, 'shan')
            for col in self.cmp_excel.ws.iter_cols():
                if col[0].value == '增':
                    for cell in col:
                        self.font_color(cell, 'zeng')

        # 如果目标工作表的关键列与原工作表的关键列不相同
        src_key_list = [i.value for idx, i in enumerate(
            self.src_excel.ws[get_column_letter(self.src_key_col)], 1) if idx > self.src_title_row]
        cmp_key_list = [i.value for idx, i in enumerate(
            self.cmp_excel.ws[get_column_letter(self.cmp_key_col)], 1) if idx > self.cmp_title_row]
        if src_key_list != cmp_key_list:
            src_key_list = duplicate_to_only(src_key_list)  # 通过重命名的方式给列表去重。
            cmp_key_list = duplicate_to_only(cmp_key_list)  # 通过重命名的方式给列表去重。
            insert_info = is_insert(src_key_list, cmp_key_list)
            delete_info = is_insert(cmp_key_list, src_key_list)
            append_info = is_appand(src_key_list, cmp_key_list)
            if insert_info != None:
                n = self.src_title_row + 1
                for i in insert_info:
                    self.src_excel.insert_rows(i + n, insert_info[i])
                    n += insert_info[i]
            if delete_info != None:
                n = self.cmp_title_row + 1
                for i in delete_info:
                    self.cmp_excel.insert_rows(i + n, delete_info[i])
                    n += delete_info[i]
            if append_info != None:
                self.src_excel.insert_rows(
                    self.src_excel.ws.max_row + 1, append_info)
            src_key_list = [i.value for idx, i in enumerate(
                self.src_excel.ws[get_column_letter(self.src_key_col)], 1) if idx > self.src_title_row]
            cmp_key_list = [i.value for idx, i in enumerate(
                self.cmp_excel.ws[get_column_letter(self.cmp_key_col)], 1) if idx > self.cmp_title_row]
            # 处理在同一位置删除行同时又插入行可能导致出现的空行问题
            for idx, i, j in zip(range(len(src_key_list)), src_key_list, cmp_key_list):
                if i == j == None:
                    r = 0  # 通过循环自加来测出在该处出现了多少空行
                    for n in range(1, idx + 1):
                        if src_key_list[idx - n] == cmp_key_list[idx - n]:
                            break
                        r += 1
                    for n in range(1, r + 1):
                        # 删除原工作表及目标工作表中的当前空行
                        self.src_excel.delete_rows(
                            self.src_title_row + idx + n)
                        self.cmp_excel.delete_rows(
                            self.cmp_title_row + idx + n)
                        # 将原工作表前一项的下方插入空白行
                        self.src_excel.insert_rows(
                            self.src_title_row + idx + 2 - n)
                        # 将对比工作表前一顶的上方插入空白行，从而形成错行的效果
                        self.cmp_excel.insert_rows(
                            self.cmp_title_row + idx + 1 - n)
        # 对比行，在原工作表行首写入标记同时设置字体颜色
        self.src_excel.insert_cols(1, width=3)  # 在原工作表首列处插入地行空列用于标记'增','删','改'
        for cell in self.src_excel.ws['A']:
            cell.alignment = Alignment(horizontal='center', vertical='center')
            cell.border = Border(left=Side('thin'), right=Side(
                'thin'), top=Side('thin'), bottom=Side('thin'))
        self.src_key_col += 1
        src_key_list = [i.value for idx, i in enumerate(
            self.src_excel.ws[get_column_letter(self.src_key_col)], 1) if idx > self.src_title_row]
        cmp_key_list = [i.value for idx, i in enumerate(
            self.cmp_excel.ws[get_column_letter(self.cmp_key_col)], 1) if idx > self.cmp_title_row]

        for idx, i, j in zip(range(self.src_title_row + 1, len(src_key_list) + self.src_title_row + 1), src_key_list, cmp_key_list):
            if i == None != j:
                self.src_excel.ws[f'A{idx}'].value = '增'
            if i != None == j:
                self.src_excel.ws[f'A{idx}'].value = '删'
        # 逐项对比数据，标注改动的地方
        for x in range(1, self.src_excel.ws.max_column):
            for y in range(2, self.src_excel.ws.max_row + 1):
                if self.src_excel.ws[f'{get_column_letter(x + 1)}{1}'].value not in ['增', '删', '临'] and self.src_excel.ws[f'A{y}'].value not in ['增', '删', '临'] and self.cmp_excel.ws[f'{get_column_letter(x)}{1}'].value not in ['增', '删', '临']:
                    if self.src_excel.ws[f'{get_column_letter(x + 1)}{y}'].value != self.cmp_excel.ws[f'{get_column_letter(x)}{y}'].value:
                        self.src_excel.ws[f'A{y}'].value = '改'
                        self.font_color(self.src_excel.ws[f'A{y}'], 'gai')
                        if self.src_excel.ws[f'{get_column_letter(x + 1)}{y}'].value == None:
                            self.font_color(
                                self.cmp_excel.ws[f'{get_column_letter(x)}{y}'], 'zeng')
                        elif self.cmp_excel.ws[f'{get_column_letter(x)}{y}'].value == None:
                            self.font_color(
                                self.src_excel.ws[f'{get_column_letter(x + 1)}{y}'], 'shan')
                        else:
                            self.font_color(
                                self.src_excel.ws[f'{get_column_letter(x + 1)}{y}'], 'gai')
                            self.font_color(
                                self.cmp_excel.ws[f'{get_column_letter(x)}{y}'], 'gai')
        # 删除临时行
        for i in range(self.src_excel.ws.max_column, 0, -1):
            if self.src_excel.ws[f'{get_column_letter(i)}1'].value == '临':
                self.src_excel.delete_cols(i)
        for i in range(self.cmp_excel.ws.max_column, 0, -1):
            if self.cmp_excel.ws[f'{get_column_letter(i)}1'].value == '临':
                self.cmp_excel.delete_cols(i)

        # 将原工作表及目标工作表合并后写入报告文件
        self.src_excel.insert_rows(1, height=20)
        self.src_excel.ws['A1'].value = '原 工 作 表'
        self.src_excel.ws['A1'].font = Font(name='微软雅黑', size=16, b=True)
        self.src_excel.ws['A1'].alignment = Alignment(
            horizontal='center', vertical='center')
        self.src_excel.ws.merge_cells(
            f'A1:{get_column_letter(self.src_excel.ws.max_column)}1')
        self.cmp_excel.insert_rows(1, height=20)
        self.cmp_excel.ws['A1'].value = '目 标 工 作 表'
        self.cmp_excel.ws['A1'].font = Font(name='微软雅黑', size=16, b=True)
        self.cmp_excel.ws['A1'].alignment = Alignment(
            horizontal='center', vertical='center')
        self.cmp_excel.ws.merge_cells(
            f'A1:{get_column_letter(self.cmp_excel.ws.max_column)}1')

        rpt = ExcelOperate(sheet_name='对比结果')
        cp = SheetCopy()
        rpt = cp.copy_sheet(src_file=self.src_excel, tag_file=rpt)
        rpt = cp.copy_sheet(src_file=self.cmp_excel, tag_file=rpt,
                            origin_col=self.src_excel.ws.max_column + 1)
        for row in rpt.ws.iter_rows():
            if row[0].value == '增':
                for cell in row:
                    self.font_color(cell, 'zeng')
            if row[0].value == '删':
                for cell in row:
                    self.font_color(cell, 'shan')
        return rpt


# 调试
if __name__ == '__main__':
    src_excel = ExcelOperate('tests\比较示例 - 原.xlsx')
    cmp_excel = ExcelOperate('tests\比较示例 - 对比.xlsx')
    report_path = 'D:/Desktop/对比报告.xlsx'
    cmper = SheetComparison(src_excel, cmp_excel)
    cmper.set_title_row(2, 2)
    cmper.set_key_col(2, 2)
    report = cmper.compare()
    report.save(report_path)
    print('对比完成！对比报告已保存至：', report_path)



# -*- encoding: utf-8 -*-
'''
@File    :   list_operate.py
@Author  :   北极星光 
@Contact :   light22@126.com
'''


def duplicate_to_only(l: list, remove=False):
    '''===================================\n
    传入一个列表，去除列表中的重复项
    l: 目标列表
    remove: 默认参数False将第二个及之后重复的项目重命名为name(1),name(2)...的格式，传入True则直接删除列表中第二次及以后出现的所有重复项。
    '''
    tmp_list = [i for i in l]  # 复制一份列表以便后续操作
    if remove == False:
        for i in range(len(l)):
            if l.count(l[i]) > 1:
                if i != l.index(l[i]):
                    for n in range(l.count(l[i])):
                        if f'{l[i]} ({n+1})' not in tmp_list:
                            tmp_list[i] = f'{l[i]} ({n+1})'
                            break
        return tmp_list
    else:
        tmp_list.reverse()
        for i in l:
            if tmp_list.count(i) > 1:
                tmp_list.remove(i)
        tmp_list.reverse()
        return tmp_list


def is_insert(srcl: list, tagl: list):
    '''===================================\n
    传入两个列表，判断目标列表是否为原列表插入元素所得。如果判断为是则返回一个以插入位置索引为键，该索引位置插入的元素个数为值的一个字典；否则返回None。
    srcl: 原列表
    tagl: 目标列表
    '''
    insert_info = {}
    for i in range(len(tagl)):
        if tagl[i] not in srcl:
            for ins in range(1, len(tagl)-i):
                if tagl[i+ins] in srcl:
                    if srcl.index(tagl[i+ins]) not in insert_info:
                        insert_info[srcl.index(tagl[i+ins])] = 1
                    else:
                        insert_info[srcl.index(tagl[i+ins])] += 1
                    break
    if insert_info == {}:
        return None
    else:
        return insert_info


def is_appand(srcl: list, tagl: list):
    '''===================================\n
    传入两个列表，判断目标列表是否为原列表后添加元素所得。如果判断为是则返回添加的元素个数；否则返回None。
    srcl: 原列表
    tagl: 目标列表
    '''
    append_info = None
    if tagl[-1] != srcl[-1] and srcl[-1] in tagl:
        append_info = len(tagl[tagl.index(srcl[-1])+1:])
    return append_info


def is_delete(srcl: list, tagl: list):
    '''===================================\n
    传入两个列表，判断目标列表是否为原列表删除元素所得。如果判断为是则返回一个以删除位置索引为键，该索引位置删除的元素个数为值的一个字典；否则返回None。
    srcl: 原列表
    tagl: 目标列表
    '''
    delete_info = {}
    for i in srcl:
        if i not in tagl:
            if srcl.index(i) not in delete_info:
                delete_info[srcl.index(i)] = 1
            else:
                delete_info[srcl.index(i)] += 1
    if delete_info == {}:
        return None
    else:
        return delete_info


if __name__ == '__main__':
    srcl = [0, 1, 2, 3, 4,None,None, 5, 6, 7, 8, 9]
    tagl = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    l = duplicate_to_only(srcl)
    print(l)